# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import DataMigration
from django.db import models
from racks.models import SOURCE_LDAP, SOURCE_LOCAL


class Migration(DataMigration):

    depends_on = (
        ("racks", "0003_fix_table_names"),
    )

    def forwards(self, orm):
        "Write your forwards methods here."
        # Note: Remember to use orm['appname.ModelName'] rather than "from appname.models..."

        ldap_source = orm['racks.source'].objects.get(source=SOURCE_LDAP)
        local_source = orm['racks.source'].objects.get(source=SOURCE_LOCAL)
        unpack = lambda row: row['username']

        # Remember which users are superusers
        superusers = set(map(unpack, orm.models['auth.user'].objects.filter(
            is_superuser=True).values('username')))

        # Retrieve local users' password hashes (just in case)
        password_hashes = dict(
            (u['username'], u['password'])
            for u in orm.models['auth.user'].objects.values('username', 'password')
        )

        # Get list of local usernames
        local_usernames = map(unpack, orm.models['auth.user'].objects.values('username'))

        # Detect which of them have matching records in RacksUser table
        racks_usernames = map(unpack, orm.models['inventory.racksuser'].objects.filter(
            username__in=local_usernames).values('username'))

        # Retrieve data for local users which have no matching records in RacksUser
        retain_usernames = set(local_usernames) - set(racks_usernames)
        local_users = list(orm.models['auth.user'].objects.filter(username__in=retain_usernames).values())

        # Cleanup User and Group tables
        orm.models['auth.user'].objects.all().delete()
        orm.models['auth.group'].objects.all().delete()

        # Transfer all records from RacksUser/RacksGroup to User/Group tables
        groups = {}
        for group in orm.models['inventory.racksgroup'].objects.filter(deleted=False):
            groups[group.id] = orm.models['auth.group'].objects.create(id=group.id, name=group.name)
            groups[group.id].sources.add(ldap_source)
        for racksuser in orm.models['inventory.racksuser'].objects.prefetch_related('groups').all():
            user = orm.models['auth.user'].objects.create(
                id=racksuser.id,
                username=racksuser.username,
                password=password_hashes.get(racksuser.username, ''),
                first_name=racksuser.first_name,
                last_name=racksuser.last_name,
                email=racksuser.email,
                is_active=not racksuser.deleted,
                is_staff=racksuser.username in superusers,
                is_superuser=racksuser.username in superusers,
            )
            for group in racksuser.groups.all():
                if not group.deleted:
                    user.groups.add(groups[group.id])
            user.sources.add(ldap_source)

        # Recreate previously remembered local users with no matches in RacksUser table
        for local_user in local_users:
            user = orm.models['auth.user'].objects.create(
                username=local_user['username'],
                password=local_user['password'],
                first_name=local_user['first_name'],
                last_name=local_user['last_name'],
                email=local_user['email'],
                is_active=local_user['is_active'],
                is_staff=local_user['is_staff'],
                is_superuser=local_user['is_superuser'],
                last_login=local_user['last_login'],
                date_joined=local_user['date_joined'],
            )
            user.sources.add(local_source)

    def backwards(self, orm):
        "Write your backwards methods here."
        orm.models['inventory.racksgroup'].objects.all().delete()
        orm.models['inventory.racksuser'].objects.all().delete()
        groups = dict((group.id, group) for group in orm.models['auth.group'].objects.all())
        for group in groups.itervalues():
            orm.models['inventory.racksgroup'].objects.create(
                id=group.id, name=group.name
            )
        for user in orm.models['auth.user'].objects.filter(sources__source=SOURCE_LDAP).prefetch_related('sources'):
            orm.models['inventory.racksuser'].objects.create(
                id=user.id,
                username=user.username,
                first_name=user.first_name,
                last_name=user.last_name,
                email=user.email,
                deleted=not user.is_active,
            )
        orm.models['auth.group'].objects.all().delete()
        orm.models['auth.user'].objects.filter(sources__source=SOURCE_LDAP).delete()


    models = {
        u'auth.group': {
            'Meta': {'object_name': 'Group'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        u'auth.permission': {
            'Meta': {'ordering': "(u'content_type__app_label', u'content_type__model', u'codename')", 'unique_together': "((u'content_type', u'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['contenttypes.ContentType']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        u'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'related_name': "u'user_set'", 'blank': 'True', 'to': u"orm['auth.Group']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'related_name': "u'user_set'", 'blank': 'True', 'to': u"orm['auth.Permission']"}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        u'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        u'inventory.facterlocation': {
            'Meta': {'object_name': 'FacterLocation'},
            'active': ('django.db.models.fields.BooleanField', [], {'default': 'True', 'db_index': 'True'}),
            'added_by': ('django.db.models.fields.CharField', [], {'max_length': '64', 'null': 'True', 'blank': 'True'}),
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'fact': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '32'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'location': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'facts'", 'null': 'True', 'on_delete': 'models.SET_NULL', 'to': u"orm['inventory.Location']"}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'})
        },
        u'inventory.inventoryobject': {
            'Meta': {'ordering': "('-id',)", 'object_name': 'InventoryObject'},
            'active': ('django.db.models.fields.BooleanField', [], {'default': 'True', 'db_index': 'True'}),
            'added_by': ('django.db.models.fields.CharField', [], {'max_length': '64', 'null': 'True', 'blank': 'True'}),
            'assigned_group': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'assigned_objects'", 'null': 'True', 'to': u"orm['inventory.RacksGroup']"}),
            'assigned_user': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'assigned_objects'", 'null': 'True', 'to': u"orm['inventory.RacksUser']"}),
            'bugs': ('django.db.models.fields.CommaSeparatedIntegerField', [], {'max_length': '254', 'null': 'True', 'blank': 'True'}),
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'dcim_id': ('django.db.models.fields.IntegerField', [], {'db_index': 'True', 'unique': 'True', 'null': 'True', 'blank': 'True'}),
            'dcim_jira_issue': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'dcim_label': ('inventory.fields.NullableCharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'facter_location': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'inventory_objects'", 'null': 'True', 'to': u"orm['inventory.FacterLocation']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'ipmi_host': ('inventory.fields.NullableCharField', [], {'max_length': '255', 'unique': 'True', 'null': 'True', 'blank': 'True'}),
            'ipmi_password': ('inventory.fields.EncryptedCharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'ipmi_user': ('inventory.fields.NullableCharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'ipv4network': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'related_name': "'inventory_objects'", 'blank': 'True', 'to': u"orm['inventory.IPv4Network']"}),
            'l2network': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'related_name': "'inventory_objects'", 'blank': 'True', 'to': u"orm['inventory.L2Network']"}),
            'label': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'related_name': "'inventory_objects'", 'blank': 'True', 'to': u"orm['inventory.InventoryObjectLabel']"}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'name': ('inventory.fields.NullableCharField', [], {'db_index': 'True', 'max_length': '254', 'unique': 'True', 'null': 'True', 'blank': 'True'}),
            'orphan_dcim_id': ('django.db.models.fields.IntegerField', [], {'db_index': 'True', 'null': 'True', 'blank': 'True'}),
            'responsible': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'related_name': "'inventory_objects'", 'blank': 'True', 'db_index': 'True', 'to': u"orm['inventory.RacksUser']"}),
            'type': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'inventory_objects'", 'null': 'True', 'to': u"orm['inventory.InventoryObjectType']"})
        },
        u'inventory.inventoryobjectattribute': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('name', 'object'),)", 'object_name': 'InventoryObjectAttribute'},
            'active': ('django.db.models.fields.BooleanField', [], {'default': 'True', 'db_index': 'True'}),
            'added_by': ('django.db.models.fields.CharField', [], {'max_length': '64', 'null': 'True', 'blank': 'True'}),
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '254'}),
            'object': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'attributes'", 'to': u"orm['inventory.InventoryObject']"}),
            'value': ('django.db.models.fields.TextField', [], {})
        },
        u'inventory.inventoryobjectcredential': {
            'Meta': {'unique_together': "(('name', 'context', 'inventory_object'),)", 'object_name': 'InventoryObjectCredential'},
            'active': ('django.db.models.fields.BooleanField', [], {'default': 'True', 'db_index': 'True'}),
            'added_by': ('django.db.models.fields.CharField', [], {'max_length': '64', 'null': 'True', 'blank': 'True'}),
            'context': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'inventory_object': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'inventory_objects'", 'to': u"orm['inventory.InventoryObject']"}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'value': ('django.db.models.fields.CharField', [], {'max_length': '255'})
        },
        u'inventory.inventoryobjectlabel': {
            'Meta': {'object_name': 'InventoryObjectLabel'},
            'active': ('django.db.models.fields.BooleanField', [], {'default': 'True', 'db_index': 'True'}),
            'added_by': ('django.db.models.fields.CharField', [], {'max_length': '64', 'null': 'True', 'blank': 'True'}),
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '64'}),
            'type': ('django.db.models.fields.CharField', [], {'default': "'GY'", 'max_length': '2'})
        },
        u'inventory.inventoryobjecttype': {
            'Meta': {'object_name': 'InventoryObjectType'},
            'active': ('django.db.models.fields.BooleanField', [], {'default': 'True', 'db_index': 'True'}),
            'added_by': ('django.db.models.fields.CharField', [], {'max_length': '64', 'null': 'True', 'blank': 'True'}),
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '254'})
        },
        u'inventory.ipv4network': {
            'Meta': {'ordering': "('address',)", 'unique_together': "(('address', 'netmask'),)", 'object_name': 'IPv4Network'},
            'active': ('django.db.models.fields.BooleanField', [], {'default': 'True', 'db_index': 'True'}),
            'added_by': ('django.db.models.fields.CharField', [], {'max_length': '64', 'null': 'True', 'blank': 'True'}),
            'address': ('django.db.models.fields.CharField', [], {'max_length': '15'}),
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'gateway': ('django.db.models.fields.CharField', [], {'max_length': '15'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'l2network': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'ipv4_networks'", 'null': 'True', 'on_delete': 'models.SET_NULL', 'to': u"orm['inventory.L2Network']"}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'netmask': ('django.db.models.fields.CharField', [], {'max_length': '15'})
        },
        u'inventory.l2address': {
            'Meta': {'object_name': 'L2Address'},
            'address': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '12'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'inventory_object': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'l2addresses'", 'null': 'True', 'to': u"orm['inventory.InventoryObject']"})
        },
        u'inventory.l2network': {
            'Meta': {'object_name': 'L2Network'},
            'active': ('django.db.models.fields.BooleanField', [], {'default': 'True', 'db_index': 'True'}),
            'added_by': ('django.db.models.fields.CharField', [], {'max_length': '64', 'null': 'True', 'blank': 'True'}),
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '254'}),
            'vlanid': ('django.db.models.fields.IntegerField', [], {})
        },
        u'inventory.lab': {
            'Meta': {'object_name': 'Lab'},
            'active': ('django.db.models.fields.BooleanField', [], {'default': 'True', 'db_index': 'True'}),
            'added_by': ('django.db.models.fields.CharField', [], {'max_length': '64', 'null': 'True', 'blank': 'True'}),
            'assigned_group': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'assigned_labs'", 'null': 'True', 'to': u"orm['inventory.RacksGroup']"}),
            'assigned_user': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'assigned_labs'", 'null': 'True', 'to': u"orm['inventory.RacksUser']"}),
            'bugs': ('django.db.models.fields.CommaSeparatedIntegerField', [], {'max_length': '254', 'null': 'True', 'blank': 'True'}),
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'inventory_objects': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'related_name': "'labs'", 'blank': 'True', 'to': u"orm['inventory.InventoryObject']"}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'responsible': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'related_name': "'labs'", 'blank': 'True', 'to': u"orm['inventory.RacksUser']"}),
            'vlans': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['inventory.L2Network']", 'symmetrical': 'False', 'blank': 'True'})
        },
        u'inventory.location': {
            'Meta': {'ordering': "('name',)", 'object_name': 'Location'},
            'active': ('django.db.models.fields.BooleanField', [], {'default': 'True', 'db_index': 'True'}),
            'added_by': ('django.db.models.fields.CharField', [], {'max_length': '64', 'null': 'True', 'blank': 'True'}),
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '254'})
        },
        u'inventory.racksgroup': {
            'Meta': {'object_name': 'RacksGroup'},
            'deleted': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'users': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'related_name': "'groups'", 'blank': 'True', 'to': u"orm['inventory.RacksUser']"})
        },
        u'inventory.racksuser': {
            'Meta': {'object_name': 'RacksUser'},
            'deleted': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'username': ('django.db.models.fields.CharField', [], {'max_length': '255'})
        },
        u'racks.source': {
            'Meta': {'object_name': 'Source'},
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'related_name': "'sources'", 'symmetrical': 'False', 'to': u"orm['auth.Group']"}),
            'source': ('django.db.models.fields.IntegerField', [], {'default': '0', 'primary_key': 'True'}),
            'users': ('django.db.models.fields.related.ManyToManyField', [], {'related_name': "'sources'", 'symmetrical': 'False', 'to': u"orm['auth.User']"})
        }
    }

    complete_apps = ['auth', 'inventory']
    symmetrical = True
