# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models
from django.db.utils import OperationalError


class Migration(SchemaMigration):
    """
    Migration patched with a workaround for South 0.7.5 bug in index name generation.

    db.create_table has a bug in index name generation, where digest is calculated from
    ([field_name], ) instead of (field_name, ). So we try to call db.delete_index with field name
    in double brackets first to force the same digest generation. However if reverse migration
    has been run, an index with a correct name will be created, so we fallback to a proper call
    as well.

    db.rename_table does not rename index names, so if a table was created with an index and later
    renamed, any later attempts to delete the index will fail as it will still have a name based on
    original table name.

    Code also accounts for possibility that this migration might have partially run already.
    """

    def forwards(self, orm):
        # Deleting field 'InventoryObject.added_by'
        try:
            db.delete_column(u'inventory_inventoryobject', 'added_by')
        except OperationalError:
            pass

        # Removing index on 'InventoryObject', fields ['active']
        try:
            db.delete_index(u'inventory_inventoryobject', [['active']])
        except OperationalError:
            db.delete_index(u'inventory_inventoryobject', ['active'])

        # Deleting field 'FacterLocation.added_by'
        db.delete_column(u'inventory_facterlocation', 'added_by')

        # Removing index on 'FacterLocation', fields ['active']
        try:
            # Rename table to it's original name, so generated index name will match
            db.rename_table(u'inventory_facterlocation', u'inventory_inventoryobjectlocation')
            try:
                db.delete_index(u'inventory_inventoryobjectlocation', [['active']])
            except OperationalError:
                db.delete_index(u'inventory_inventoryobjectlocation', ['active'])
            finally:
                db.rename_table(u'inventory_inventoryobjectlocation', u'inventory_facterlocation')
        except OperationalError:
            db.delete_index(u'inventory_facterlocation', ['active'])

        # Deleting field 'InventoryObjectAttribute.added_by'
        db.delete_column(u'inventory_inventoryobjectattribute', 'added_by')

        # Removing index on 'InventoryObjectAttribute', fields ['active']
        try:
            db.delete_index(u'inventory_inventoryobjectattribute', [['active']])
        except OperationalError:
            db.delete_index(u'inventory_inventoryobjectattribute', ['active'])

        # Deleting field 'InventoryObjectType.created'
        db.delete_column(u'inventory_inventoryobjecttype', 'created')

        # Deleting field 'InventoryObjectType.modified'
        db.delete_column(u'inventory_inventoryobjecttype', 'modified')

        # Deleting field 'InventoryObjectType.active'
        db.delete_column(u'inventory_inventoryobjecttype', 'active')

        # Deleting field 'InventoryObjectType.added_by'
        db.delete_column(u'inventory_inventoryobjecttype', 'added_by')

        # Deleting field 'InventoryObjectLabel.added_by'
        db.delete_column(u'inventory_inventoryobjectlabel', 'added_by')

        # Removing index on 'InventoryObjectLabel', fields ['active']
        try:
            db.delete_index(u'inventory_inventoryobjectlabel', [['active']])
        except OperationalError:
            db.delete_index(u'inventory_inventoryobjectlabel', ['active'])

        # Deleting field 'Location.added_by'
        db.delete_column(u'inventory_location', 'added_by')

        # Removing index on 'Location', fields ['active']
        try:
            db.delete_index(u'inventory_location', [['active']])
        except OperationalError:
            db.delete_index(u'inventory_location', ['active'])

        # Deleting field 'Lab.added_by'
        db.delete_column(u'inventory_lab', 'added_by')

        # Removing index on 'Lab', fields ['active']
        try:
            db.delete_index(u'inventory_lab', [['active']])
        except OperationalError:
            db.delete_index(u'inventory_lab', ['active'])

        # Deleting field 'InventoryObjectCredential.added_by'
        db.delete_column(u'inventory_inventoryobjectcredential', 'added_by')

        # Removing index on 'InventoryObjectCredential', fields ['active']
        try:
            db.delete_index(u'inventory_inventoryobjectcredential', [['active']])
        except OperationalError:
            db.delete_index(u'inventory_inventoryobjectcredential', ['active'])

        # Deleting field 'IPv4Network.added_by'
        db.delete_column(u'inventory_ipv4network', 'added_by')

        # Removing index on 'IPv4Network', fields ['active']
        try:
            db.delete_index(u'inventory_ipv4network', [['active']])
        except OperationalError:
            db.delete_index(u'inventory_ipv4network', ['active'])

        # Deleting field 'L2Network.added_by'
        db.delete_column(u'inventory_l2network', 'added_by')

        # Removing index on 'L2Network', fields ['active']
        try:
            db.delete_index(u'inventory_l2network', [['active']])
        except OperationalError:
            db.delete_index(u'inventory_l2network', ['active'])


    def backwards(self, orm):
        # Adding index on 'L2Network', fields ['active']
        db.create_index(u'inventory_l2network', ['active'])

        # Adding index on 'IPv4Network', fields ['active']
        db.create_index(u'inventory_ipv4network', ['active'])

        # Adding index on 'InventoryObjectCredential', fields ['active']
        db.create_index(u'inventory_inventoryobjectcredential', ['active'])

        # Adding index on 'Lab', fields ['active']
        db.create_index(u'inventory_lab', ['active'])

        # Adding index on 'Location', fields ['active']
        db.create_index(u'inventory_location', ['active'])

        # Adding index on 'InventoryObjectLabel', fields ['active']
        db.create_index(u'inventory_inventoryobjectlabel', ['active'])

        # Adding index on 'InventoryObjectAttribute', fields ['active']
        db.create_index(u'inventory_inventoryobjectattribute', ['active'])

        # Adding index on 'FacterLocation', fields ['active']
        db.create_index(u'inventory_facterlocation', ['active'])

        # Adding index on 'InventoryObject', fields ['active']
        db.create_index(u'inventory_inventoryobject', ['active'])

        # Adding field 'InventoryObject.added_by'
        db.add_column(u'inventory_inventoryobject', 'added_by',
                      self.gf('django.db.models.fields.CharField')(max_length=64, null=True, blank=True),
                      keep_default=False)

        # Adding field 'FacterLocation.added_by'
        db.add_column(u'inventory_facterlocation', 'added_by',
                      self.gf('django.db.models.fields.CharField')(max_length=64, null=True, blank=True),
                      keep_default=False)

        # Adding field 'InventoryObjectAttribute.added_by'
        db.add_column(u'inventory_inventoryobjectattribute', 'added_by',
                      self.gf('django.db.models.fields.CharField')(max_length=64, null=True, blank=True),
                      keep_default=False)

        # Adding field 'InventoryObjectType.created'
        db.add_column(u'inventory_inventoryobjecttype', 'created',
                      self.gf('django.db.models.fields.DateTimeField')(auto_now_add=True, default=None, blank=True),
                      keep_default=False)

        # Adding field 'InventoryObjectType.modified'
        db.add_column(u'inventory_inventoryobjecttype', 'modified',
                      self.gf('django.db.models.fields.DateTimeField')(auto_now=True, default=None, blank=True),
                      keep_default=False)

        # Adding field 'InventoryObjectType.active'
        db.add_column(u'inventory_inventoryobjecttype', 'active',
                      self.gf('django.db.models.fields.BooleanField')(default=True, db_index=True),
                      keep_default=False)

        # Adding field 'InventoryObjectType.added_by'
        db.add_column(u'inventory_inventoryobjecttype', 'added_by',
                      self.gf('django.db.models.fields.CharField')(max_length=64, null=True, blank=True),
                      keep_default=False)

        # Adding field 'InventoryObjectLabel.added_by'
        db.add_column(u'inventory_inventoryobjectlabel', 'added_by',
                      self.gf('django.db.models.fields.CharField')(max_length=64, null=True, blank=True),
                      keep_default=False)

        # Adding field 'Location.added_by'
        db.add_column(u'inventory_location', 'added_by',
                      self.gf('django.db.models.fields.CharField')(max_length=64, null=True, blank=True),
                      keep_default=False)

        # Adding field 'Lab.added_by'
        db.add_column(u'inventory_lab', 'added_by',
                      self.gf('django.db.models.fields.CharField')(max_length=64, null=True, blank=True),
                      keep_default=False)

        # Adding field 'InventoryObjectCredential.added_by'
        db.add_column(u'inventory_inventoryobjectcredential', 'added_by',
                      self.gf('django.db.models.fields.CharField')(max_length=64, null=True, blank=True),
                      keep_default=False)

        # Adding field 'IPv4Network.added_by'
        db.add_column(u'inventory_ipv4network', 'added_by',
                      self.gf('django.db.models.fields.CharField')(max_length=64, null=True, blank=True),
                      keep_default=False)

        # Adding field 'L2Network.added_by'
        db.add_column(u'inventory_l2network', 'added_by',
                      self.gf('django.db.models.fields.CharField')(max_length=64, null=True, blank=True),
                      keep_default=False)


    models = {
        u'auth.group': {
            'Meta': {'object_name': 'Group'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        u'auth.permission': {
            'Meta': {'ordering': "(u'content_type__app_label', u'content_type__model', u'codename')", 'unique_together': "((u'content_type', u'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['contenttypes.ContentType']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        u'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'related_name': "u'user_set'", 'blank': 'True', 'to': u"orm['auth.Group']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'related_name': "u'user_set'", 'blank': 'True', 'to': u"orm['auth.Permission']"}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        u'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        u'inventory.facterlocation': {
            'Meta': {'object_name': 'FacterLocation'},
            'active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'fact': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '32'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'location': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'facts'", 'null': 'True', 'on_delete': 'models.SET_NULL', 'to': u"orm['inventory.Location']"}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'})
        },
        u'inventory.inventoryobject': {
            'Meta': {'ordering': "('-id',)", 'object_name': 'InventoryObject'},
            'active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'assigned_group': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'assigned_objects'", 'null': 'True', 'to': u"orm['auth.Group']"}),
            'assigned_user': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'assigned_objects'", 'null': 'True', 'to': u"orm['auth.User']"}),
            'bugs': ('django.db.models.fields.CommaSeparatedIntegerField', [], {'max_length': '254', 'null': 'True', 'blank': 'True'}),
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'dcim_id': ('django.db.models.fields.IntegerField', [], {'db_index': 'True', 'unique': 'True', 'null': 'True', 'blank': 'True'}),
            'dcim_jira_issue': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'dcim_label': ('inventory.fields.NullableCharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'facter_location': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'inventory_objects'", 'null': 'True', 'to': u"orm['inventory.FacterLocation']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'ipmi_host': ('inventory.fields.NullableCharField', [], {'max_length': '255', 'unique': 'True', 'null': 'True', 'blank': 'True'}),
            'ipmi_password': ('inventory.fields.EncryptedCharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'ipmi_user': ('inventory.fields.NullableCharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'ipv4network': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'related_name': "'inventory_objects'", 'blank': 'True', 'to': u"orm['inventory.IPv4Network']"}),
            'l2network': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'related_name': "'inventory_objects'", 'blank': 'True', 'to': u"orm['inventory.L2Network']"}),
            'label': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'related_name': "'inventory_objects'", 'blank': 'True', 'to': u"orm['inventory.InventoryObjectLabel']"}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'name': ('inventory.fields.NullableCharField', [], {'db_index': 'True', 'max_length': '254', 'unique': 'True', 'null': 'True', 'blank': 'True'}),
            'orphan_dcim_id': ('django.db.models.fields.IntegerField', [], {'db_index': 'True', 'null': 'True', 'blank': 'True'}),
            'responsible': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'related_name': "'inventory_objects'", 'blank': 'True', 'db_index': 'True', 'to': u"orm['auth.User']"}),
            'type': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'inventory_objects'", 'null': 'True', 'to': u"orm['inventory.InventoryObjectType']"})
        },
        u'inventory.inventoryobjectattribute': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('name', 'object'),)", 'object_name': 'InventoryObjectAttribute'},
            'active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '254'}),
            'object': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'attributes'", 'to': u"orm['inventory.InventoryObject']"}),
            'value': ('django.db.models.fields.TextField', [], {})
        },
        u'inventory.inventoryobjectcredential': {
            'Meta': {'unique_together': "(('name', 'context', 'inventory_object'),)", 'object_name': 'InventoryObjectCredential'},
            'active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'context': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'inventory_object': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'inventory_objects'", 'to': u"orm['inventory.InventoryObject']"}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'value': ('django.db.models.fields.CharField', [], {'max_length': '255'})
        },
        u'inventory.inventoryobjectlabel': {
            'Meta': {'object_name': 'InventoryObjectLabel'},
            'active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '64'}),
            'type': ('django.db.models.fields.CharField', [], {'default': "'GY'", 'max_length': '2'})
        },
        u'inventory.inventoryobjecttype': {
            'Meta': {'object_name': 'InventoryObjectType'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '254'})
        },
        u'inventory.ipv4network': {
            'Meta': {'ordering': "('address',)", 'unique_together': "(('address', 'netmask'),)", 'object_name': 'IPv4Network'},
            'active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'address': ('django.db.models.fields.CharField', [], {'max_length': '15'}),
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'gateway': ('django.db.models.fields.CharField', [], {'max_length': '15'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'l2network': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'ipv4_networks'", 'null': 'True', 'on_delete': 'models.SET_NULL', 'to': u"orm['inventory.L2Network']"}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'netmask': ('django.db.models.fields.CharField', [], {'max_length': '15'})
        },
        u'inventory.l2address': {
            'Meta': {'object_name': 'L2Address'},
            'address': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '12'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'inventory_object': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'l2addresses'", 'null': 'True', 'to': u"orm['inventory.InventoryObject']"})
        },
        u'inventory.l2network': {
            'Meta': {'object_name': 'L2Network'},
            'active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '254'}),
            'vlanid': ('django.db.models.fields.IntegerField', [], {})
        },
        u'inventory.lab': {
            'Meta': {'object_name': 'Lab'},
            'active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'assigned_group': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'assigned_labs'", 'null': 'True', 'to': u"orm['auth.Group']"}),
            'assigned_user': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'assigned_labs'", 'null': 'True', 'to': u"orm['auth.User']"}),
            'bugs': ('django.db.models.fields.CommaSeparatedIntegerField', [], {'max_length': '254', 'null': 'True', 'blank': 'True'}),
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'inventory_objects': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'related_name': "'labs'", 'blank': 'True', 'to': u"orm['inventory.InventoryObject']"}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'responsible': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'related_name': "'labs'", 'blank': 'True', 'to': u"orm['auth.User']"}),
            'vlans': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['inventory.L2Network']", 'symmetrical': 'False', 'blank': 'True'})
        },
        u'inventory.location': {
            'Meta': {'ordering': "('name',)", 'object_name': 'Location'},
            'active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '254'})
        }
    }

    complete_apps = ['inventory']