from django import forms

from dcim.models import SwitchPort, DCIM_SERVER
from inventory.models import Lab
from racks import crud


class LabDetailView(crud.DetailView):

    def get_context_data(self, **kwargs):
        context = super(LabDetailView, self).get_context_data(**kwargs)
        switch_ports = SwitchPort.objects.filter(
            dcim_device_type=DCIM_SERVER,
            l2addresses__inventory_object_id__in=[
                io.pk for io in self.object.inventory_objects.all()
            ]
        ).select_related('switch').prefetch_related('l2addresses__inventory_object')
        switches = {}
        for port in switch_ports:
            if port.switch_id not in switches:
                switches[port.switch_id] = {'switch': port.switch, 'ports': []}
            switches[port.switch_id]['ports'].append(port)
        context['switches'] = switches.values()
        return context


class LabViewSet(crud.ViewSet):
    model = Lab
    detail_view = LabDetailView
    filtered_select_fields = ('vlans', 'responsible', 'inventory_objects')
    widgets = {
        'bugs': forms.TextInput(attrs={'data-role': 'bugsinput'})
    }
    fields = '__all__'


urlpatterns = LabViewSet('inventory.labs').urlpatterns
