from django.db.models import Count

from dcim.models import Switch, DCIM_SERVER, DCIM_SWITCH
from racks import crud


def _port_keyfunc(p):
    try:
        return int(p.number)
    except (ValueError, TypeError):
        return p.number


class SwitchListView(crud.ListView):

    def get_queryset(self):
        return Switch.objects.prefetch_related(
            'ports__l2addresses__inventory_object__facter_location__location'
        ).annotate(
            l2addresses=Count('ports__l2addresses__id', distinct=True),
            inventory_objects=Count('ports__l2addresses__inventory_object__id', distinct=True),
        ).order_by('label')


class SwitchDetailView(crud.DetailView):

    def get_context_data(self, **kwargs):
        context = super(SwitchDetailView, self).get_context_data(**kwargs)

        object = Switch.objects.prefetch_related(
            'ports__l2addresses__inventory_object__facter_location__location'
        ).get(pk=self.object.pk)
        sorted_ports = sorted(object.ports.all(), key=_port_keyfunc)

        context.update({
            'object': object,
            'sorted_ports': sorted_ports,
            'DCIM_SWITCH': DCIM_SWITCH,
            'DCIM_SERVER': DCIM_SERVER,
        })
        return context


class SwitchesViewSet(crud.ViewSet):
    model = Switch
    change_form_class = None
    list_view = SwitchListView
    detail_view = SwitchDetailView
    create_view = None
    delete_view = None
    update_view = None


urlpatterns = SwitchesViewSet('inventory.switches').urlpatterns
