# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models
from django.db.utils import OperationalError

class Migration(SchemaMigration):
    """
    Migration patched with a workaround for South 0.7.5 bug in index name generation.

    db.create_table has a bug in index name generation, where digest is calculated from
    ([field_name], ) instead of (field_name, ). So we try to call db.delete_index with field name
    in double brackets first to force the same digest generation. However if reverse migration
    has been run, an index with a correct name will be created, so we fallback to a proper call
    as well.

    Code also accounts for possibility that this migration might have partially run already.
    """

    def forwards(self, orm):
        # Deleting field 'JenkinsNode.added_by'
        try:
            db.delete_column(u'jenkins_ci_jenkinsnode', 'added_by')
        except OperationalError:
            # If migration has run before, 'added_by' is already deleted, so we skip this exception
            pass

        # Removing index on 'JenkinsNode', fields ['active']
        try:
            db.delete_index(u'jenkins_ci_jenkinsnode', [['active']])
        except OperationalError:
            db.delete_index(u'jenkins_ci_jenkinsnode', ['active'])

        # Deleting field 'JenkinsInstance.added_by'
        db.delete_column(u'jenkins_ci_jenkinsinstance', 'added_by')

        # Removing index on 'JenkinsInstance', fields ['active']
        try:
            db.delete_index(u'jenkins_ci_jenkinsinstance', [['active']])
        except OperationalError:
            db.delete_index(u'jenkins_ci_jenkinsinstance', ['active'])


    def backwards(self, orm):
        # Adding index on 'JenkinsInstance', fields ['active']
        db.create_index(u'jenkins_ci_jenkinsinstance', ['active'])

        # Adding index on 'JenkinsNode', fields ['active']
        db.create_index(u'jenkins_ci_jenkinsnode', ['active'])

        # Adding field 'JenkinsNode.added_by'
        db.add_column(u'jenkins_ci_jenkinsnode', 'added_by',
                      self.gf('django.db.models.fields.CharField')(max_length=64, null=True, blank=True),
                      keep_default=False)

        # Adding field 'JenkinsInstance.added_by'
        db.add_column(u'jenkins_ci_jenkinsinstance', 'added_by',
                      self.gf('django.db.models.fields.CharField')(max_length=64, null=True, blank=True),
                      keep_default=False)


    models = {
        u'auth.group': {
            'Meta': {'object_name': 'Group'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        u'auth.permission': {
            'Meta': {'ordering': "(u'content_type__app_label', u'content_type__model', u'codename')", 'unique_together': "((u'content_type', u'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['contenttypes.ContentType']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        u'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'related_name': "u'user_set'", 'blank': 'True', 'to': u"orm['auth.Group']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'related_name': "u'user_set'", 'blank': 'True', 'to': u"orm['auth.Permission']"}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        u'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        u'inventory.facterlocation': {
            'Meta': {'object_name': 'FacterLocation'},
            'active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'fact': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '32'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'location': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'facts'", 'null': 'True', 'on_delete': 'models.SET_NULL', 'to': u"orm['inventory.Location']"}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'})
        },
        u'inventory.inventoryobject': {
            'Meta': {'ordering': "('-id',)", 'object_name': 'InventoryObject'},
            'active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'assigned_group': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'assigned_objects'", 'null': 'True', 'to': u"orm['auth.Group']"}),
            'assigned_user': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'assigned_objects'", 'null': 'True', 'to': u"orm['auth.User']"}),
            'bugs': ('django.db.models.fields.CommaSeparatedIntegerField', [], {'max_length': '254', 'null': 'True', 'blank': 'True'}),
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'dcim_id': ('django.db.models.fields.IntegerField', [], {'db_index': 'True', 'unique': 'True', 'null': 'True', 'blank': 'True'}),
            'dcim_jira_issue': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'dcim_label': ('inventory.fields.NullableCharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'facter_location': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'inventory_objects'", 'null': 'True', 'to': u"orm['inventory.FacterLocation']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'ipmi_host': ('inventory.fields.NullableCharField', [], {'max_length': '255', 'unique': 'True', 'null': 'True', 'blank': 'True'}),
            'ipmi_password': ('inventory.fields.EncryptedCharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'ipmi_user': ('inventory.fields.NullableCharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'ipv4network': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'related_name': "'inventory_objects'", 'blank': 'True', 'to': u"orm['inventory.IPv4Network']"}),
            'l2network': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'related_name': "'inventory_objects'", 'blank': 'True', 'to': u"orm['inventory.L2Network']"}),
            'label': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'related_name': "'inventory_objects'", 'blank': 'True', 'to': u"orm['inventory.InventoryObjectLabel']"}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'name': ('inventory.fields.NullableCharField', [], {'db_index': 'True', 'max_length': '254', 'unique': 'True', 'null': 'True', 'blank': 'True'}),
            'orphan_dcim_id': ('django.db.models.fields.IntegerField', [], {'db_index': 'True', 'null': 'True', 'blank': 'True'}),
            'responsible': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'related_name': "'inventory_objects'", 'blank': 'True', 'db_index': 'True', 'to': u"orm['auth.User']"}),
            'type': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'inventory_objects'", 'null': 'True', 'to': u"orm['inventory.InventoryObjectType']"})
        },
        u'inventory.inventoryobjectlabel': {
            'Meta': {'object_name': 'InventoryObjectLabel'},
            'active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '64'}),
            'type': ('django.db.models.fields.CharField', [], {'default': "'GY'", 'max_length': '2'})
        },
        u'inventory.inventoryobjecttype': {
            'Meta': {'object_name': 'InventoryObjectType'},
            'active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '254'})
        },
        u'inventory.ipv4network': {
            'Meta': {'ordering': "('address',)", 'unique_together': "(('address', 'netmask'),)", 'object_name': 'IPv4Network'},
            'active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'address': ('django.db.models.fields.CharField', [], {'max_length': '15'}),
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'gateway': ('django.db.models.fields.CharField', [], {'max_length': '15'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'l2network': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'ipv4_networks'", 'null': 'True', 'on_delete': 'models.SET_NULL', 'to': u"orm['inventory.L2Network']"}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'netmask': ('django.db.models.fields.CharField', [], {'max_length': '15'})
        },
        u'inventory.l2network': {
            'Meta': {'object_name': 'L2Network'},
            'active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '254'}),
            'vlanid': ('django.db.models.fields.IntegerField', [], {})
        },
        u'inventory.location': {
            'Meta': {'ordering': "('name',)", 'object_name': 'Location'},
            'active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '254'})
        },
        u'jenkins_ci.jenkinserrorlog': {
            'Meta': {'object_name': 'JenkinsErrorLog'},
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'instance': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'error_logs'", 'null': 'True', 'on_delete': 'models.SET_NULL', 'to': u"orm['jenkins_ci.JenkinsInstance']"}),
            'instance_name': ('django.db.models.fields.CharField', [], {'max_length': '32'}),
            'instance_url': ('django.db.models.fields.CharField', [], {'max_length': '127'}),
            'message': ('django.db.models.fields.CharField', [], {'max_length': '255'})
        },
        u'jenkins_ci.jenkinsinstance': {
            'Meta': {'object_name': 'JenkinsInstance'},
            'active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'api_key': ('inventory.fields.EncryptedCharField', [], {'default': 'None', 'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'last_failed_sync': ('django.db.models.fields.DateTimeField', [], {'default': 'None', 'null': 'True'}),
            'last_successful_sync': ('django.db.models.fields.DateTimeField', [], {'default': 'None', 'null': 'True'}),
            'master_node': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'+'", 'null': 'True', 'on_delete': 'models.SET_NULL', 'to': u"orm['jenkins_ci.JenkinsNode']"}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '32'}),
            'url': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '255'}),
            'username': ('inventory.fields.NullableCharField', [], {'default': 'None', 'max_length': '63', 'null': 'True', 'blank': 'True'})
        },
        u'jenkins_ci.jenkinslabel': {
            'Meta': {'object_name': 'JenkinsLabel'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '63'})
        },
        u'jenkins_ci.jenkinsnode': {
            'Meta': {'object_name': 'JenkinsNode'},
            'active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'address': ('django.db.models.fields.CharField', [], {'max_length': '15', 'null': 'True'}),
            'architecture': ('inventory.fields.NullableCharField', [], {'default': 'None', 'max_length': '63', 'null': 'True'}),
            'available_memory': ('django.db.models.fields.BigIntegerField', [], {'default': '0'}),
            'available_swap': ('django.db.models.fields.BigIntegerField', [], {'default': '0'}),
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'disabled': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'disk_space': ('django.db.models.fields.BigIntegerField', [], {'default': '0'}),
            'executors': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'hostname': ('django.db.models.fields.CharField', [], {'max_length': '127', 'null': 'True', 'db_index': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'instance': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'nodes'", 'null': 'True', 'on_delete': 'models.SET_NULL', 'to': u"orm['jenkins_ci.JenkinsInstance']"}),
            'inventory_object': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'jenkins_nodes'", 'null': 'True', 'on_delete': 'models.SET_NULL', 'to': u"orm['inventory.InventoryObject']"}),
            'labels': ('django.db.models.fields.related.ManyToManyField', [], {'related_name': "'nodes'", 'symmetrical': 'False', 'to': u"orm['jenkins_ci.JenkinsLabel']"}),
            'last_job': ('django.db.models.fields.DateTimeField', [], {'default': 'None', 'null': 'True'}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '127', 'db_index': 'True'}),
            'offline': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'offline_reason': ('inventory.fields.NullableCharField', [], {'default': 'None', 'max_length': '127', 'null': 'True'}),
            'temp_disk_space': ('django.db.models.fields.BigIntegerField', [], {'default': '0'}),
            'temporarily_offline': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'total_memory': ('django.db.models.fields.BigIntegerField', [], {'default': '0'}),
            'total_swap': ('django.db.models.fields.BigIntegerField', [], {'default': '0'})
        }
    }

    complete_apps = ['jenkins_ci']