from django.conf import settings

from racks import crud

from jenkins_ci.forms import JenkinsNodeForm
from jenkins_ci.models import JenkinsNode


class JenkinsNodeListView(crud.ListView):
    template_name = 'jenkins_ci/nodes/change_list.html'
    display_actions = not settings.JENKINS_DISABLE_UI_FORMS

    def get_queryset(self):
        # Fixed the query which was silently accepted in 1.6 but generates a warning in 1.8
        return JenkinsNode.objects.select_related('instance').prefetch_related('labels')\
            .filter(**self.get_filters()).order_by('instance__name', 'name')

    def get_filters(self):
        label = self.request.GET.get('filter_label')
        return {'labels__name': label} if label else {}

    def get_context_data(self, **kwargs):
        context = super(JenkinsNodeListView, self).get_context_data(**kwargs)
        label = self.request.GET.get('filter_label')
        if label:
            context['get_parameters'] = 'filter_label=%s' % label
            context['filter_label'] = label
        return context


class JenkinsNodeDetailView(crud.DetailView):
    template_name = 'jenkins_ci/nodes/detail.html'


class JenkinsNodeViewSet(crud.ViewSet):
    model = JenkinsNode
    list_view = JenkinsNodeListView
    detail_view = JenkinsNodeDetailView
    create_view = None
    delete_view = None
    if settings.JENKINS_DISABLE_UI_FORMS:
        update_view = None
        change_form_class = None
    else:
        change_form_class = JenkinsNodeForm

    readonly_fields = ('name', 'hostname', 'address')


urlpatterns = JenkinsNodeViewSet('jenkins_ci.nodes').urlpatterns
