from django.db import models
from django.contrib.auth.models import User, Group
from django.utils.encoding import python_2_unicode_compatible

SOURCE_LOCAL = 0
SOURCE_LDAP = 1
SOURCES_CHOICES = (
    (SOURCE_LOCAL, 'Local'),
    (SOURCE_LDAP, 'LDAP')
)


class RacksModel(models.Model):
    "Ancestor to all Racks models, used when some changes to Meta must be propagated everywhere"

    class Meta:
        abstract = True


class RacksTimestampedModel(RacksModel):
    created = models.DateTimeField(auto_now_add=True)
    modified = models.DateTimeField(auto_now=True)
    active = models.BooleanField(default=True)

    class Meta:
        abstract = True


@python_2_unicode_compatible
class Source(RacksModel):
    source = models.IntegerField(choices=SOURCES_CHOICES, default=SOURCE_LOCAL, primary_key=True)
    # user and group set as ManyToMany if user/group was imported from
    # multiple sources, and should be deleted only if in all sources it was
    # removed and not to override User and Group models from contrib
    users = models.ManyToManyField(User, related_name='sources')
    groups = models.ManyToManyField(Group, related_name='sources')

    def __str__(self):
        return str(SOURCES_CHOICES[self.source][1])

    def __repr__(self):
        return "<Source: %s>" % str(SOURCES_CHOICES[self.source][1])
